@@ -24,7 +24,7 @@ impl Default for Gender {
     }
 }
 
-#[derive(Debug, Deserialize, Clone, Default)]
+#[derive(Debug, Deserialize, Serialize, Clone, Default)]
 pub struct Armor {
     pub name: String,
     pub skills: Vec<(Skill, u8)>,
