@@ -1,16 +1,17 @@
+use itertools::Itertools;
+use serde::{Deserialize, Serialize};
 use std::{
     array,
     cmp::{min, Ordering},
+    collections::HashMap,
     iter,
 };
 
 use crate::armor_and_skills::{Armor, Gender, Skill};
 
-use itertools::Itertools;
-
 pub type Jewels = [Option<Skill>; 3];
 
-#[derive(Debug)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Build {
     pub helmet: Option<(Armor, Jewels)>,
     pub chest: Option<(Armor, Jewels)>,
@@ -21,6 +22,46 @@ pub struct Build {
     pub weapon_jewels: Jewels,
 }
 
+impl Build {
+    pub fn get_all_skills_and_amounts(&self) -> HashMap<Skill, u8> {
+        let mut hm = HashMap::with_capacity(5);
+        for opt in array::IntoIter::new([
+            &self.helmet,
+            &self.chest,
+            &self.arm,
+            &self.waist,
+            &self.leg,
+            &self.talisman,
+        ]) {
+            if let Some((armor, jewels)) = opt {
+                for (skill, amount) in armor.skills.iter() {
+                    if !hm.contains_key(skill) {
+                        hm.insert(*skill, 0);
+                    }
+                    *hm.get_mut(skill).unwrap() += amount
+                }
+                for jewel in jewels.iter() {
+                    if let Some(skill) = jewel {
+                        if !hm.contains_key(skill) {
+                            hm.insert(*skill, 0);
+                        }
+                        *hm.get_mut(skill).unwrap() += 1
+                    }
+                }
+            }
+        }
+        for jewel in self.weapon_jewels.iter() {
+            if let Some(skill) = jewel {
+                if !hm.contains_key(skill) {
+                    hm.insert(*skill, 0);
+                }
+                *hm.get_mut(skill).unwrap() += 1
+            }
+        }
+        hm
+    }
+}
+
 /// Returns a Vec containing every item of the provided slice but wrapped in an Option, then add None
 /// at the end of the Vec.
 /// # Why
